; flash sequence 5 LEDs on GP0, GP1, GP2, GP4, and GP5 using stored patterns

	list      p=12f629            		; list directive to define processor
	#include <p12f629.inc>        		; processor specific variable definitions

	errorlevel  -302              		; suppress message 302 from list file

	__CONFIG   _CP_OFF & _CPD_OFF & _BODEN_OFF & _MCLRE_OFF & _WDT_OFF & _PWRTE_ON & _INTRC_OSC_NOCLKOUT  

; '__CONFIG' directive is used to embed configuration word within .asm file.
; The lables following the directive are located in the respective .inc file.
; See data sheet for additional information on configuration word settings.

;***** VARIABLE DEFINITIONS

pointer		EQU		0x20
flags		EQU		0x21
counter		EQU		0x22
again		EQU		0x23
position	EQU		0x24
position2	EQU		0x25

loop1		EQU		0x30
loop2		EQU		0x31
cnt1		EQU		0x32
cnt2		EQU		0x33


;***** CONSTANTS

p1			EQU		10
p2			EQU		6
p3			EQU		1
p4			EQU		10
p5			EQU		10
p6			EQU		10
lp1			EQU		4
lp2			EQU		3
pos1		EQU		0
pos2		EQU		9
pos3		EQU		11
pos4		EQU		12
pos5		EQU		17
pos6		EQU		22


;***** MACROS

bank0:  MACRO 
        bcf STATUS, RP0 
        ENDM 

bank1:  MACRO 
        bsf STATUS, RP0 
        ENDM 


;***** PROGRAM CODE

		ORG     0x000					; processor reset vector

		goto    Main					; go to beginning of program


		ORG     0x004					; interrupt vector location

Interrupt
		bcf		INTCON, T0IF

		decfsz	loop1
		goto	IntDone

		movf	cnt1, W
		movwf	loop1

		decfsz	loop2
		goto	IntDone
		
		movf	cnt2, W
		movwf	loop2

		bsf		flags, 0

IntDone
		retfie							; return from interrupt

Main

		bank1
		
; set TRISIO, all output except GP3
		movlw	b'001000'
		movwf	TRISIO

; use TMR0
		
		bsf		OPTION_REG, NOT_GPPU	; disable internal global pullups
		bcf		OPTION_REG, INTEDG		; int of edgge of GP2 (not used anyway)
		bcf		OPTION_REG, T0CS		; use internal osc
		bcf		OPTION_REG, T0SE		; (not used anyway)
		bcf		OPTION_REG, PSA			; prescalar assignment
		bsf		OPTION_REG, PS2			; scalar \
		bcf		OPTION_REG, PS1			; scalar  } x64
		bsf		OPTION_REG, PS0			; scalar /

		bank0

		movlw	lp1
		movwf	loop1
		movwf	cnt1
		movlw	lp2
		movwf	loop2
		movwf	cnt2

		bcf		INTCON, T0IF			; clear the tmr0 interrupt flag
		bsf		INTCON, T0IE			; enable tmr0 interrupts
		bcf		INTCON, PEIE			; disable other interrupts
		bcf		INTCON, INTE			;
		bcf		INTCON, GPIE			;
		bsf		INTCON, GIE				; enable enalbed interrupts

; main loop

Loop
		clrf	pointer
		clrf	position
		clrf	GPIO

Part1
		movlw	p1
		movwf	counter
		movlw	pos2
		movwf	position2

		movlw	pos1
		call	DoPattern

Part2
		movlw	p2
		movwf	counter
		movlw	pos3
		movwf	position2

		movlw	pos2
		call	DoPattern

Part3
		movlw	p3
		movwf	counter
		movlw	pos4
		movwf	position2

		movlw	pos3
		call	DoPattern

Part4
		movlw	p4
		movwf	counter
		movlw	pos5
		movwf	position2

		movlw	pos4
		call	DoPattern

        goto    Loop 

Part5
		movlw	p5
		movwf	counter
		movlw	pos6
		movwf	position2

		movlw	pos5
		call	DoPattern

        goto    Loop 


; **** subroutines

DoPattern
		movwf	position
		movwf	again

Again
		movf	again, W
		movwf	position

SLoop									; set the output LEDs
		movf	position, W
		call	LookUp
		movwf	GPIO
		incf	position, F

Delay									; wait for interrupt flag
		btfss	flags, 0
		goto	Delay
		bcf		flags, 0

		movf	position, W				; has the end of the pattern been hit?
		subwf	position2, W
		btfss	STATUS, Z
		goto	SLoop					;  - no so process the next
	
		decfsz	counter, F				; repeat pattern?
        goto	Again

Done
		return


		;ORG	0x100						; patterns for LEDs

LookUp
		addwf	PCL, F
		retlw	b'000001'				; pattern 1 (0-8)
		retlw	b'000010'
		retlw	b'000100'
		retlw	b'010000'
		retlw	b'100000'
		retlw	b'010000'
		retlw	b'000100'
		retlw	b'000010'
		retlw	b'000001'

		retlw	b'000000'				; pattern 2 (9-10)
		retlw	b'110111'

		retlw	b'110011'				; pattern 3 (11)

		retlw	b'100001'				; pattern 4 (12-16)
		retlw	b'010010'
		retlw	b'000100'
		retlw	b'010010'
		retlw	b'100001'

		retlw	b'000100'				; pattern 5 (17-21)
		retlw	b'010110'
		retlw	b'010010'
		retlw	b'110011'
		retlw	b'100001'

		retlw	b'000000'				; (22)

		END                  	     	; directive 'end of program'

